/****************************************************************************
**
** Copyright (C) 2006-2009 fullmetalcoder <fullmetalcoder@hotmail.fr>
**
** This file is part of the Edyuk project <http://edyuk.org>
** 
** This file may be used under the terms of the GNU General Public License
** version 3 as published by the Free Software Foundation and appearing in the
** file GPL.txt included in the packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#ifndef _QDOCUMENT_REGION_TREE_H_
#define _QDOCUMENT_REGION_TREE_H_

#include "qce-config.h"

#include <QMap>
#include <QVector>

class QDocumentPrivate;

class QCE_EXPORT QDocumentRegionTree
{
	public:
		enum 
		{
			MinimumRegionSize = 256,
			AverageRegionSize = 2048,
			MaximumRegionSize = 4096,
		};
		
		QDocumentRegionTree(QDocumentPrivate *d);
		
		void reset(int lineCount);
		void clearSpans(bool defer = false);
		
		void performDeferredUpdates() const;
		
		void linesRemoved(int logicalLine, int count, bool defer = false);
		void linesInserted(int logicalLine, int count, bool defer = false);
		
		void hide(int logicalLine, int count, bool defer = false);
		void show(int logicalLine, int count, bool defer = false);
		
		void setSpan(int logicalLine, int count, bool defer = false);
		
		int visualLineNumber(int logicalLine) const;
		int logicalLineNumber(int visualLine, int *wrap = 0) const;
		
		int visualLineCount() const;
		
		struct Region
		{
			Region() {}
			~Region() {}
			
			void updateOffsets(int from);
			
			int visualLine(int logicalLine) const;
			int logicalLine(int visualLine, int *wrap = 0) const;
			
			bool dirty;
			
			int logicalStart;
			int logicalCount;
			
			int visualStart;
			int visualCount;
			
			QMap<int, int> m_hidden;
			QMap<int, int> m_wrapped;
			QMap<int, int> m_offsetsTV;
			QMap<int, int> m_offsetsVT;
		};
		
	private:
		void fixRegionsFrom(int lastValidRegion, int maxRegion = -1);
		
		int visualRegion(int line) const;
		int logicalRegion(int line) const;
		
		mutable bool m_hasDeferred;
		mutable int m_lastVisual, m_lastLogical, m_firstDirtyRegion;
		
		QDocumentPrivate *d;
		QVector<Region*> m_regions;
};

#endif // !_QDOCUMENT_REGION_TREE_H_
