/****************************************************************************
**
** Copyright (C) 2006-2009 fullmetalcoder <fullmetalcoder@hotmail.fr>
**
** This file is part of the Edyuk project <http://edyuk.org>
** 
** This file may be used under the terms of the GNU General Public License
** version 3 as published by the Free Software Foundation and appearing in the
** file GPL.txt included in the packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#ifndef _QNFA_DEFINITION_H_
#define _QNFA_DEFINITION_H_

/*!
	\file qnfadefinition.h
	\brief Definition of the QNFADefinition class.
*/

#include "qlanguagefactory.h"
#include "qlanguagedefinition.h"

#include <QHash>
#include <QPointer>
#include <QStringList>

struct QNFA;

template <typename T>
class QStack;

class QFile;
class QDomElement;
class QDomNodeList;
class QDomDocument;

class QNFAAction
{
	public:
		enum
		{
			NoAction	= 0,
			
			Ambiguous	= 0x40000000,
			
			Content		= 0x80000000
		};
};

class QCE_EXPORT QNFADefinition : public QLanguageDefinition
{
	public:
		QNFADefinition();
		virtual ~QNFADefinition();
		
		virtual QString language() const;
		virtual QStringList extensions() const;
		
		virtual int tokenize(QDocument *d, int line, int count);
		
		virtual QString singleLineComment() const;
		
		virtual QString defaultLineMark() const;
		
		virtual void clearMatches(QDocument *d);
		virtual void match(QDocumentCursor& c);
		
		virtual QString indent(const QDocumentCursor& c);
		virtual bool unindent (const QDocumentCursor& c, const QString& ktxt);
		
		virtual void expand(QDocument *d, int line);
		virtual void collapse(QDocument *d, int line);
		virtual int blockFlags(QDocument *d, int line, int depth) const;
		
		virtual int tokenFormat(int tokenId) const;
		virtual int contextFormat(int contextId) const;
		
		static void load(QFile *f, QLanguageFactory::LangData *d, QFormatScheme *s);
		static void load(const QString& file, QLanguageFactory::LangData *d, QFormatScheme *s);
		static void load(const QDomDocument& doc, QLanguageFactory::LangData *d, QFormatScheme *s);
		
	protected:
		int blockFlags(const QVector<int>& tokens) const;
		void tokenize(
						QDocumentLineHandle *h,
						QNFA*& context, QStack<QNFA*>& cxt,
						QStack<int>& cxtidx, QMap<int, QPair<int, int> >& cxtData
					);
		
		int action(QDomElement c, QFormatScheme *f, int fid = 0);
		
		void fillContext(QNFA *cxt, QDomElement e, QFormatScheme *f, bool cs);
		void fillContext(QNFA *cxt, QDomNodeList l, QFormatScheme *f, bool cs);
		
		void addToContext(QNFA *cxt, QDomElement c, int fid,
					QFormatScheme *f,
					const QStringList& pref,
					const QStringList& suff,
					bool cs);
		
		int& addContext(const QString& id, QNFA *nfa);
		void addEmbedRequest(const QString& lang, QNFA *dest);
		void shareEmbedRequests(QNFA *src, QNFA *dest, int offset);
		
	private:
		bool m_indentFold;
		QString m_language,
				m_defaultMark,
				m_singleLineComment;
		
		QStringList m_extensions;
		
		QNFA *m_root;
		
		struct Token
		{
			int flags;
			int format;
			int parenId;
			QLanguageDefinition *d;
		};
		
		struct Context
		{
			int id;
			int format;
			QNFA *nfa;
			QLanguageDefinition *d;
		};
		
		struct Paren
		{
			class Matcher
			{
				public:
					virtual ~Matcher() {}
					virtual bool match(const QString& p1, const QString& p2) const = 0; 
			};
			
			int id;
			Matcher *matcher;
		};
		
		static QList<Token> m_tokens;
		static QList<Paren> m_parentheses;
		static QHash<int, Context> m_contexts;
		static QHash<QNFA*, int> m_contextIds;
		
		QHash<QPointer<QDocument>, int> m_matchGroups;
		
		static QHash<QString, int> m_paren;
		static QHash<QString, QNFA*> m_contextNames;
		
		struct PMatch
		{
			PMatch() : type(Invalid)
			{
				line[0] = -1;
				line[1] = -1;
				
				column[0] = -1;
				column[1] = -1;
				
				length[0] = 0;
				length[1] = 0;
			}
			
			enum Type
			{
				Invalid,
				Match,
				Mismatch
			};
			
			char type;
			
			int line[2];
			int column[2];
			int length[2];
		};
		
		void matchOpen(QDocument *d, PMatch& m);
		void matchClose(QDocument *d, PMatch& m);
		
		int findBlockEnd(QDocument *d, int line, bool *open = 0);
		
		
		static void flushEmbedRequests(const QString& lang);
		
		struct EmbedRequest
		{
			inline EmbedRequest(QNFA *nfa, int idx) : index(idx), target(nfa) {}
			
			int index;
			QNFA *target;
		};
		
		typedef QList<EmbedRequest> EmbedRequestList;
		
		static QHash<QString, EmbedRequestList> m_pendingEmbeds;
};

#endif // !_QNFA_DEFINITION_H_
